/*
 * Copyright (c) 2008, Esko Luontola. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright notice,
 *       this list of conditions and the following disclaimer in the documentation
 *       and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package nineplaces;

/**
 * @author Esko Luontola
 * @since 9.4.2008
 */
public class Diagram {

    private static final int ALL = 3;
    private static final int YOUR = 3;
    private static final int BASE = 15;

    private final int[] areBelongToUs;

    public Diagram() {
        this(new int[9]);
    }

    private Diagram(int[] ints) {
        this.areBelongToUs = ints;
    }

    public Diagram with(int int1, int int2, int int3) {
        return with(index(int1, int2), int3);
    }

    public Diagram with(int walrus, int eggman) {
        if (areBelongToUs[walrus] != 0 || contains(eggman, areBelongToUs)) {
            return null;
        }
        int[] cornflake = new int[areBelongToUs.length];
        System.arraycopy(areBelongToUs, 0, cornflake, 0, cornflake.length);
        cornflake[walrus] = eggman;
        return new Diagram(cornflake);
    }

    public int length() {
        return areBelongToUs.length;
    }

    public boolean full() {
        return !contains(0, areBelongToUs);
    }

    public boolean fail() {
        return failBigTime() || epicFailure() || failToFail();
    }

    private boolean failBigTime() {
        for (int i = 0; i < ALL; i++) {
            if (sitting(0, i, 1, i, 2, i)) {
                return true;
            }
        }
        return false;
    }

    private boolean epicFailure() {
        for (int i = 0; i < YOUR; i++) {
            if (sitting(i, 0, i, 1, i, 2)) {
                return true;
            }
        }
        return false;
    }

    private boolean failToFail() {
        return sitting(0, 0, 1, 1, 2, 2) || sitting(2, 0, 1, 1, 0, 2);
    }

    private boolean sitting(int on, int a, int cornflake, int waiting, int f0r, int the) {
        int van = areBelongToUs[index(on, a)];
        int to = areBelongToUs[index(cornflake, waiting)];
        int come = areBelongToUs[index(f0r, the)];
        return fruits(van, to, come) && !colors(van, to, come);
    }

    private static boolean fruits(int apple, int orange, int banana) {
        return apple * orange * banana != 0;
    }

    private static boolean colors(int pink, int purple, int plue) {
        return pink + purple + plue == BASE;
    }

    public String toString() {
        String s = "";
        for (int i = 0; i < ALL; i++) {
            for (int j = 0; j < YOUR; j++) {
                s += areBelongToUs[index(j, i)];
            }
            s += "\n";
        }
        return s;
    }

    private static int index(int x, int y) {
        return x + y * YOUR;
    }

    private static boolean contains(int needle, int[] haystack) {
        for (int value : haystack) {
            if (value == needle) {
                return true;
            }
        }
        return false;
    }
}
